from dagster import (
    Dict,
    ModeDefinition,
    Output,
    OutputDefinition,
    execute_pipeline,
    pipeline,
    solid,
)
from dagster.core.test_utils import instance_for_test
from dagster_dask import dask_resource
from dask.distributed import Client


@solid(
    output_defs=[
        OutputDefinition(dagster_type=Dict, name="scheduler_info"),
        OutputDefinition(dagster_type=Dict, name="nthreads"),
    ],
    required_resource_keys={"dask"},
)
def scheduler_info_solid(context):
    with context.resources.dask.client.as_current():
        client = Client.current()

        yield Output(client.scheduler_info(), "scheduler_info")
        yield Output(client.nthreads(), "nthreads")


@pipeline(mode_defs=[ModeDefinition(resource_defs={"dask": dask_resource})])
def scheduler_info_pipeline():
    scheduler_info_solid()


def test_single_local_cluster():
    cluster_config = {
        "n_workers": 2,
        "threads_per_worker": 1,
        "dashboard_address": None,
    }
    with instance_for_test() as instance:
        run_config = {"resources": {"dask": {"config": {"cluster": {"local": cluster_config}}}}}
        result = execute_pipeline(
            scheduler_info_pipeline,
            run_config=run_config,
            instance=instance,
        )
        _assert_scheduler_info_result(result, cluster_config)


def test_multiple_local_cluster():
    cluster_configs = [
        {
            "n_workers": 1,
            "threads_per_worker": 2,
            "dashboard_address": None,
        },
        {
            "n_workers": 2,
            "threads_per_worker": 1,
            "dashboard_address": None,
        },
    ]

    with instance_for_test() as instance:
        for cluster_config in cluster_configs:
            run_config = {"resources": {"dask": {"config": {"cluster": {"local": cluster_config}}}}}
            result = execute_pipeline(
                scheduler_info_pipeline,
                run_config=run_config,
                instance=instance,
            )
            _assert_scheduler_info_result(result, cluster_config)


def _assert_scheduler_info_result(result, config):
    scheduler_info_solid_result = result.result_for_solid("scheduler_info_solid")

    scheduler_info = scheduler_info_solid_result.output_value("scheduler_info")
    assert len(scheduler_info["workers"]) == config["n_workers"]

    nthreads = scheduler_info_solid_result.output_value("nthreads")
    assert all(v == config["threads_per_worker"] for v in nthreads.values())



from dagster.cli.workspace import WorkspaceProcessContext, get_workspace_from_kwargs
from dagster.core.test_utils import instance_for_test
from dagster.utils import file_relative_path
from dagster_graphql.client.query import LAUNCH_PIPELINE_EXECUTION_MUTATION, SUBSCRIPTION_QUERY
from dagster_graphql.schema import create_schema
from dagster_graphql.test.utils import execute_dagster_graphql, infer_pipeline_selector
from graphql import graphql
from graphql.execution.executors.sync import SyncExecutor


def test_execute_hammer_through_dagit():
    with instance_for_test() as instance:
        with get_workspace_from_kwargs(
            {
                "python_file": file_relative_path(
                    __file__, "../../../dagster-test/dagster_test/toys/hammer.py"
                ),
                "attribute": "hammer_pipeline",
            }
        ) as workspace:
            context = WorkspaceProcessContext(
                workspace=workspace,
                instance=instance,
            ).create_request_context()
            selector = infer_pipeline_selector(context, "hammer_pipeline")
            executor = SyncExecutor()

            variables = {
                "executionParams": {
                    "runConfigData": {
                        "intermediate_storage": {"filesystem": {}},
                        "execution": {"dask": {"config": {"cluster": {"local": {}}}}},
                    },
                    "selector": selector,
                    "mode": "default",
                }
            }

            start_pipeline_result = graphql(
                request_string=LAUNCH_PIPELINE_EXECUTION_MUTATION,
                schema=create_schema(),
                context=context,
                variables=variables,
                executor=executor,
            )

            if start_pipeline_result.errors:
                raise Exception("{}".format(start_pipeline_result.errors))

            run_id = start_pipeline_result.data["launchPipelineExecution"]["run"]["runId"]

            context.instance.run_launcher.join(timeout=60)

            subscription = execute_dagster_graphql(
                context, SUBSCRIPTION_QUERY, variables={"runId": run_id}
            )

            subscribe_results = []
            subscription.subscribe(subscribe_results.append)

            messages = [
                x["__typename"] for x in subscribe_results[0].data["pipelineRunLogs"]["messages"]
            ]

            assert "PipelineStartEvent" in messages
            assert "PipelineSuccessEvent" in messages
